from dataclasses import dataclass
from typing import Any, Callable

from PyQt6.QtCore import pyqtSignal
from PyQt6.QtWidgets import (
    QDialog,
    QDialogButtonBox,
    QDoubleSpinBox,
    QHBoxLayout,
    QLabel,
    QLayout,
    QLineEdit,
    QPushButton,
    QSpinBox,
    QVBoxLayout,
    QWidget,
)

from PyQt6 import QtWidgets

from jbqt.models import DialogOptions


def get_widget(name: str) -> QWidget | None:
    widget: QWidget | None = None
    if hasattr(QtWidgets, name):
        widget = getattr(QtWidgets, name)
    """ elif hasattr(widgets, name):
        widget = getattr(widgets, name) """

    if widget:
        if not callable(widget):
            return

        widget = widget()
        if isinstance(widget, QWidget):
            return widget
    return None


def get_widget_val(widget: QWidget) -> str | int | float | None:
    # Emit the custom signal with data when the dialog is accepted
    value = None
    if isinstance(widget, QLineEdit):
        value = widget.text()
    elif isinstance(widget, QSpinBox):
        value = widget.value()
    elif isinstance(widget, QDoubleSpinBox):
        value = widget.value()

    return value


class InputFormDialog(QDialog):
    """reference value for external use"""

    value = pyqtSignal(dict)

    def __init__(
        self,
        parent=None,
        form_data: list[dict] | None = None,
        opts: DialogOptions | None = None,
    ):
        super().__init__(parent)

        opts = opts or DialogOptions(title="Input")
        opts.apply(self)

        self.form_data: list[dict] = [obj.copy() for obj in form_data or []]
        self.input_widgets: dict[str, QWidget] = {}

        self.main_layout = QVBoxLayout()
        self.construct_widgets()

        button_box = QHBoxLayout()
        submit_btn = QPushButton("Submit", parent=self)
        submit_btn.clicked.connect(self.submit)

        cancel_btn = QPushButton("Cancel", parent=self)
        cancel_btn.clicked.connect(self.close)

        button_box.addWidget(submit_btn)
        button_box.addWidget(cancel_btn)
        self.main_layout.addLayout(button_box)

        self.setLayout(self.main_layout)

    def construct_widgets(self) -> None:
        for widget_def in self.form_data:
            if not "type" in widget_def:
                continue

            layout = QHBoxLayout()

            w_type = widget_def.pop("type")
            label = widget_def.pop("label", "")
            key = widget_def.pop("key", "")
            if not key:
                continue

            widget = get_widget(w_type)
            if not widget:
                continue
            if label and isinstance(label, str):
                layout.addWidget(QLabel(label))

            if widget_def:
                self._init_widget(widget, **widget_def)
            layout.addWidget(widget)
            self.main_layout.addLayout(layout)
            self.input_widgets[key] = widget

    def get_form_data(self) -> dict:
        form_values: dict[str, Any] = {}

        for key, widget in self.input_widgets.items():
            value = get_widget_val(widget)
            if value:
                form_values[key] = value

        return form_values

    def submit(self) -> None:
        form_data = self.get_form_data()
        self.value.emit(form_data)
        self.close()

    def close(self) -> bool:
        self.deleteLater()

        return super().close()

    def _init_widget(self, widget: QWidget, **opts) -> None:
        for key, value in opts.items():
            setter_key = f"set{key[0].upper() + key[1:]}"
            if hasattr(widget, setter_key):
                setter = getattr(widget, setter_key)
                if setter:
                    setter(value)



import argparse
import os
import sys

from PyQt6.QtCore import QSize, Qt, QTimer
from PyQt6.QtGui import QIcon
from PyQt6.QtWidgets import (
    QApplication,
    QGridLayout,
    QLabel,
    QLineEdit,
    QMainWindow,
    QPushButton,
    QScrollArea,
    QStyle,
    QTableWidget,
    QTableWidgetItem,
    QVBoxLayout,
    QWidget,
)

from fuzzywuzzy import process  # Needs fuzzywuzzy + python-Levenshtein

parser = argparse.ArgumentParser(description="Simple tool to display icons")
parser.add_argument(
    "--from-theme",
    "-f",
    action="store_true",
    help="Search and display all icons from the Ubuntu themes as opposed to the StandardIcon library in Qt6",
)
parser.add_argument(
    "--theme-icons",
    "-t",
    action="store_true",
    help="Pull icons from the QIcon.ThemeIcon list",
)
parser.add_argument(
    "--add-quotes",
    "-q",
    action="store_true",
    help="Wrap copied icon name with quotes",
)
args = parser.parse_args()

THEMES_DIR = "/usr/share/icons"


# TODO: fix type checking
class Toast(QLabel):
    def __init__(self, text: str, parent: QWidget = None, duration_ms: int = 1500):  # type: ignore
        super().__init__(parent)

        self.setText(text)
        self.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.setStyleSheet(
            """
            background-color: rgba(50, 50, 50, 180);
            color: white;
            border-radius: 8px;
            padding: 8px 16px;
            font-size: 14px;
        """
        )
        self.setWindowFlags(Qt.WindowType.ToolTip)
        self.adjustSize()

        # Center it over parent if available
        if parent:
            parent_rect = parent.rect()
            toast_rect = self.rect()
            self.move(
                parent_rect.center().x() - toast_rect.width() // 2,
                parent_rect.top() + 50,  # appear slightly under the top
            )

        QTimer.singleShot(duration_ms, self.close)


def get_theme_icons(theme_dir: str) -> list[str]:
    icon_names = set()
    for root, _, files in os.walk(theme_dir):
        for file in files:
            if file.endswith((".svg", ".png")):
                rel_path = os.path.relpath(os.path.join(root, file), theme_dir)
                parts = rel_path.split(os.sep)
                if len(parts) >= 2:
                    icon_name = os.path.splitext(parts[-1])[0]
                    icon_names.add(icon_name)
    return sorted(icon_names)


class Window(QMainWindow):
    def __init__(self):
        super().__init__()

        self.setGeometry(1000, 400, 1600, 800)

        central_widget = QWidget()
        self.setCentralWidget(central_widget)

        self.layout = QVBoxLayout()  # type: ignore
        central_widget.setLayout(self.layout)  # type: ignore

        self.search_bar = QLineEdit()
        self.search_bar.setPlaceholderText("Search icons...")
        self.search_bar.textChanged.connect(self.update_display)

        self.table = QTableWidget()
        self.table.setColumnCount(4)  # 4 columns of icons
        self.table.horizontalHeader().setVisible(False)  # type: ignore
        self.table.verticalHeader().setVisible(False)  # type: ignore
        self.table.setIconSize(QSize(64, 64))
        self.table.setShowGrid(False)
        self.table.setEditTriggers(QTableWidget.EditTrigger.NoEditTriggers)
        self.table.setSelectionMode(QTableWidget.SelectionMode.NoSelection)
        self.table.setFocusPolicy(Qt.FocusPolicy.NoFocus)
        self.table.cellClicked.connect(self.copy_icon_name_to_clipboard)
        self.table.setStyleSheet(
            """
        QTableWidget::item:hover {
            background-color: #444444;
        }
        """
        )

        self.layout.addWidget(self.search_bar)  # type: ignore
        self.layout.addWidget(self.table)  # type: ignore

        cur_theme = QIcon.themeName()
        self.icon_src = "standard"
        if args.from_theme:
            self.icon_src = "from_theme"
        elif args.theme_icons:
            self.icon_src = "theme_icons"

        match self.icon_src:
            case "from_theme":
                self.icons = get_theme_icons(os.path.join(THEMES_DIR, cur_theme))
            case "theme_icons":
                self.icons = [
                    i for i in dir(QIcon.ThemeIcon) if not i.startswith("_")
                ]
            case _:
                self.icons = sorted(
                    [
                        attr
                        for attr in dir(QStyle.StandardPixmap)
                        if attr.startswith("SP_")
                    ]
                )

        self.update_display()

    def copy_icon_name_to_clipboard(self, row: int, column: int) -> None:
        item = self.table.item(row, column)
        if item and item.text():
            text = f'"{item.text()}"' if args.add_quotes else item.text()
            QApplication.clipboard().setText(text)  # type: ignore
            print(f"Copied '{item.text()}' to clipboard")
            Toast(f"Copied '{item.text()}'", parent=self).show()

    def update_display(self):
        text = self.search_bar.text().strip()

        if text:
            # Use fuzzy matching
            matches = process.extract(
                text,
                self.icons,
                limit=200,  # Don't explode the table even if 2400+ icons exist
            )
            filtered_icons = [match[0] for match in matches if match[1] >= 50]
        else:
            filtered_icons = self.icons

        self.table.setRowCount((len(filtered_icons) + 3) // 4)

        for i in range(self.table.rowCount()):
            for j in range(4):
                idx = i * 4 + j
                if idx >= len(filtered_icons):
                    self.table.setItem(i, j, QTableWidgetItem())
                    continue

                name = filtered_icons[idx]

                match self.icon_src:
                    case "from_theme":
                        icon = QIcon.fromTheme(name)
                    case "theme_icons":
                        icon = QIcon.fromTheme(getattr(QIcon.ThemeIcon, name))
                    case _:

                        pixmapi = getattr(QStyle.StandardPixmap, name)
                        icon = self.style().standardIcon(pixmapi)  # type: ignore

                item = QTableWidgetItem(name)
                item.setIcon(icon)
                self.table.setItem(i, j, item)

        self.table.resizeRowsToContents()
        self.table.resizeColumnsToContents()


app = QApplication(sys.argv)
w = Window()
w.show()

app.exec()



"""Model for chips widget"""

from PyQt6.QtWidgets import QWidget


class IChipButton(QWidget):
    """ChipButton model"""

    def __init__(self, parent: QWidget | None = None) -> None:
        super().__init__(parent)



"""Collection of constants and global references for the jbqt package."""

import os
import re

from typing import Optional


from PyQt6.QtWidgets import QApplication, QMainWindow, QScrollArea, QWidget
from PyQt6.QtCore import Qt, QSize
from PyQt6.QtGui import QIcon, QPixmap, QPainter, QColor

from jbqt.types import IconGetter

TAG_RE = re.compile(r"^\((.+)\)(\d+\.\d+)$")
LIST_ITEM_ROLE = Qt.ItemDataRole.UserRole - 1


class QtGlobalRefs:
    """Class that tracks various values globally for a given application"""

    app: QApplication | None = None
    main_window: QMainWindow | None = None
    scroll_area: QScrollArea | None = None
    server_active: bool = False
    debug_set: bool = False
    seed_widget: QWidget | None = None


class QtPaths:
    """Global path reference storage for JbQt projects"""

    icon_dir: str | None = None


THEME_ICONS: dict[str, QIcon] = {
    "save": QIcon.fromTheme("media-floppy-symbolic"),
    "new_file": QIcon.fromTheme("document-new-symbolic"),
    "open": QIcon.fromTheme("folder-open-symbolic"),
    "copy": QIcon.fromTheme("edit-paste-symbolic"),
    "inspect": QIcon.fromTheme("docviewer-app-symbolic"),
    "clone": QIcon.fromTheme("edit-copy-symbolic"),
    "plus": QIcon.fromTheme("list-add-symbolic"),
    "minus": QIcon.fromTheme("list-remove-symbolic"),
    "times": QIcon.fromTheme("cancel-operation-symbolic"),
    "refresh": QIcon.fromTheme("view-refresh-symbolic"),
    "reload": QIcon.fromTheme("update-symbolic"),
    "circle_check": QIcon.fromTheme("selection-checked"),
    "circle_x": QIcon.fromTheme("application-exit"),
    "code": QIcon.fromTheme("input-tablet-symbolic"),
    "font-selection-editor": QIcon.fromTheme("font-select-symbolic"),
    "trash": QIcon.fromTheme("trash-symbolic"),
    "edit_data": QIcon.fromTheme("document-edit-symbolic"),
    "preview": QIcon.fromTheme("view-layout-symbolic"),
    "sync": QIcon.fromTheme("mail-send-receive-symbolic"),
}
""" Mapping of system fallback icons if STD_ICONS doesn't have one"""

STD_ICONS: dict[str, QIcon] = {
    "save": QIcon.fromTheme(QIcon.ThemeIcon.DocumentSave),
    "new_file": QIcon.fromTheme(QIcon.ThemeIcon.DocumentNew),
    "open": QIcon.fromTheme(QIcon.ThemeIcon.FolderOpen),
    "copy": QIcon.fromTheme(QIcon.ThemeIcon.EditPaste),
    "inspect": QIcon.fromTheme(QIcon.ThemeIcon.DocumentPageSetup),
    "clone": QIcon.fromTheme(QIcon.ThemeIcon.EditCopy),
    "plus": QIcon.fromTheme(QIcon.ThemeIcon.ListAdd),
    "minus": QIcon.fromTheme(QIcon.ThemeIcon.DialogError),
    "times": QIcon.fromTheme(QIcon.ThemeIcon.ApplicationExit),
    "refresh": QIcon.fromTheme(QIcon.ThemeIcon.SystemReboot),
    "reload": QIcon.fromTheme(QIcon.ThemeIcon.MediaPlaylistRepeat),
    "circle_check": QIcon.fromTheme(QIcon.ThemeIcon.MediaPlaybackStart),
    "circle_x": QIcon.fromTheme(QIcon.ThemeIcon.ProcessStop),
    "code": QIcon.fromTheme(QIcon.ThemeIcon.Computer),
    "font-selection-editor": QIcon.fromTheme(QIcon.ThemeIcon.EditFind),
    "trash": QIcon.fromTheme(QIcon.ThemeIcon.EditDelete),
    "edit_data": QIcon.fromTheme(QIcon.ThemeIcon.InputTablet),
    "preview": QIcon.fromTheme(QIcon.ThemeIcon.ZoomIn),
    "sync": QIcon.fromTheme(QIcon.ThemeIcon.MediaPlaylistShuffle),
}
""" Mapping of preferred icon definitions """


def set_icon_dir(path: str) -> None:
    """Convenience function to set the global icon directory"""
    QtPaths.icon_dir = path


class QtIconSizes:
    """Reference class to retrieve common PyQt icon size classes"""

    ICON_XS = QSize(16, 16)
    ICON_SM = QSize(20, 20)
    ICON_MD = QSize(22, 22)
    ICON_LG = QSize(24, 24)
    ICON_XL = QSize(32, 32)
    ICON_XXL = QSize(48, 48)

    @classmethod
    def get(cls, key: str, default: QSize | None = None) -> QSize:
        if hasattr(cls, key):
            return getattr(cls, key)
        return default or cls.ICON_MD


def icon_dir(file_name: str) -> str:
    """Retrieve the filepath to an icon image

    Args:
        file_name (str): File name of the icon image to retrieve

    Returns:
        str: Either the global icon directory joined with file_name if
            the global icon directory has been defined, otherwise just
            the file_name
    """

    if QtPaths.icon_dir:
        return os.path.join(QtPaths.icon_dir, file_name)
    return file_name


def recolor_icon(
    image_path: str, color: str | QColor, scale: QSize | None = None
) -> QIcon:
    """Define QIcon instance and adjust the color accordingly

    Args:
        image_path (str): Path to the image file to use, or a standard
            system/theme reference string
        color (str | QColor): Color value to set the icon to
        scale (QSize | None, optional): Size to make the icon. If None,
            gets set 0to 22px. Defaults to None.

    Returns:
        QIcon: QIcon instance with the provided size and color
    """

    if not color:
        color = QColor("black")
    if isinstance(color, str):
        color = QColor(color)

    if scale is None:
        scale = QtIconSizes.ICON_MD
    pixmap = QPixmap(image_path)
    icon_key = os.path.splitext(os.path.basename(image_path))[0]

    if pixmap.isNull():
        fallback = THEME_ICONS.get(icon_key, STD_ICONS.get(icon_key))
        if fallback:
            pixmap = fallback.pixmap(scale)

    recolored_pixmap = QPixmap(pixmap.size())

    recolored_pixmap.fill(
        QColor("transparent")
    )  # Ensure the background is transparent
    painter = QPainter(recolored_pixmap)
    painter.setCompositionMode(QPainter.CompositionMode.CompositionMode_Source)

    # Draw the original pixmap
    painter.drawPixmap(0, 0, pixmap)

    # Apply the desired color
    painter.setCompositionMode(QPainter.CompositionMode.CompositionMode_SourceIn)
    painter.fillRect(pixmap.rect(), color)

    painter.end()
    return QIcon(recolored_pixmap.scaled(scale, Qt.AspectRatioMode.KeepAspectRatio))


def get_icon(file_name: str) -> IconGetter:
    """Wrapper function that returns a function to build a QIcon instance

    Args:
        file_name (str): Icon image file name in the global icon directory,
            or a standard system/theme icon string

    Returns:
        IconGetter: Function that builds the actual QIcon instance based
            on the provided file name
    """

    def getter(
        color: Optional[str | QColor] = "", size: QSize | int | None = None
    ) -> QIcon:
        color = color or ""
        if isinstance(size, int):
            size = QSize(size, size)

        return recolor_icon(icon_dir(file_name), color, size)

    return getter


class ICONS:
    """Reference class that provides common predefined IconGetter functions"""

    SAVE = get_icon("save.svg")
    NEW = get_icon("new_file.svg")
    OPEN = get_icon("open.png")
    COPY = get_icon("copy.png")
    INSPECT = get_icon("inspect.svg")
    CLONE = get_icon("clone.svg")
    PLUS = get_icon("plus.png")
    MINUS = get_icon("minus.png")
    TIMES = get_icon("times.svg")
    REFRESH = get_icon("refresh.svg")
    RELOAD = get_icon("reload.png")
    CIRCLE_CHECK = get_icon("circle_check.svg")
    CIRCLE_TIMES = get_icon("circle_x.svg")
    CODE = get_icon("code.png")
    EDIT = get_icon("font-selection-editor.png")
    TRASH = get_icon("trash.png")
    EDIT_DATA = get_icon("edit_data.svg")
    PREVIEW = get_icon("preview.png")
    SYNC = get_icon("sync.png")

    @classmethod
    def get(cls, name: str, default: str = "") -> IconGetter:
        """Function to retrieve an IconGetter reference by name.

        Args:
            name (str): _description_
            default (str, optional): _description_. Defaults to "".

        Returns:
            IconGetter | str: _description_
        """
        if hasattr(cls, name):
            return getattr(cls, name)
        else:
            return get_icon(default)


__all__ = [
    "ICONS",
    "QtIconSizes",
]



from PyQt6.QtGui import QFontMetrics
from PyQt6.QtWidgets import (
    QDialog,
    QVBoxLayout,
    QTextEdit,
    QPushButton,
)


class TextPreviewDialog(QDialog):
    def __init__(self, text: str, title: str = "", parent=None):
        super().__init__(parent)

        self.setWindowTitle("Text Preview")
        if title:
            self.setWindowTitle(f"Text Preview [{title}]")

        layout = QVBoxLayout(self)

        # QTextEdit to display the text
        self.text_edit = QTextEdit(self)
        self.text_edit.setPlainText(text)
        self.text_edit.setReadOnly(True)
        layout.addWidget(self.text_edit)

        # Close button
        close_button = QPushButton("Close", self)
        close_button.clicked.connect(self.accept)
        layout.addWidget(close_button)

        # Adjust the dialog size based on the text
        self.adjust_size_based_on_text(text)

    def adjust_size_based_on_text(self, text):
        # Calculate text size
        font_metrics = QFontMetrics(self.text_edit.font())
        text_size = font_metrics.size(0, text)

        # Adjust size with some padding
        h_padding = 20
        v_padding = 100
        self.resize(text_size.width() + h_padding, text_size.height() + v_padding)



from dataclasses import dataclass

from PyQt6.QtCore import QObject
from PyQt6.QtWidgets import QWidget, QMainWindow, QApplication, QDialog


@dataclass
class DialogOptions:
    title: str = ""
    x_pos: int | None = None
    y_pos: int | None = None
    height: int | None = None
    width: int | None = None

    def apply(self, q_obj: QWidget | QMainWindow | QDialog):
        if self.title:
            q_obj.setWindowTitle(self.title)

        h, w, x, y = self.height, self.width, self.x_pos, self.y_pos
        if h or w:
            h = h or 400
            w = w or 400
            q_obj.resize(w, h)

        if x is not None or y is not None:
            x = x or 0
            y = y or 0
            q_obj.move(x, y)



"""Collection of widget exports"""


from jbqt.widgets.chip_button import ChipButton
from jbqt.widgets.chips import ChipsWidget
from jbqt.widgets.multiselect import MultiSelectComboBox
from jbqt.widgets.simple import ClickableLabel, LongIntSpinBox
from jbqt.widgets.toast import Toast

WIDGET_LIST = [ChipButton, ChipsWidget, MultiSelectComboBox, ClickableLabel, LongIntSpinBox, Toast]


__all__ = [
    "ChipButton",
    "ChipsWidget",
    "MultiSelectComboBox",
    "ClickableLabel",
    "LongIntSpinBox",
    "Toast",
]



"""Collection of utility functions local to the models package"""

from jbutils import JbuConsole
from jbutils.types import Function
from rich import print

from jbqt.models.model_consts import RegisteredFunctions


def register_fct(name: str, fct: Function, override: bool = False) -> None:
    exists = name in RegisteredFunctions

    if not exists or override:
        RegisteredFunctions[name] = fct
        if exists:
            JbuConsole.warn(
                f"Function '{name}' already defined and was overwritten"
            )
    else:
        JbuConsole.warn(f"Function '{name}' already defined")


def get_fct(name: str, fallback: Function | None = None) -> Function | None:
    return RegisteredFunctions.get(name, fallback)


__all__ = ["register_fct", "get_fct"]



from PyQt6.QtCore import pyqtSignal
from PyQt6.QtWidgets import QWidget, QPushButton, QVBoxLayout, QFileDialog


class JbFileDialog(QWidget):
    """A simple widget that opens a file dialog in either 'open' or 'save' mode.

    Emits:
        selectedFile (str): Emitted when a file path is chosen.

    Args:
        title (str): The dialog window title.
        directory (str): The starting directory.
        mode (str): 'open' to open an existing file, 'save' to choose a save location.
    """

    selectedFile = pyqtSignal(str)

    def __init__(
        self, title: str = "File Dialog", directory: str = "", mode: str = "open"
    ) -> None:
        super().__init__()

        self.title = title
        self.directory = directory
        self.mode = mode.lower()

        if self.mode not in {"open", "save"}:
            raise ValueError("mode must be either 'open' or 'save'")

        self.setWindowTitle(title)

        self.button = QPushButton(
            "Open File" if self.mode == "open" else "Save File", self
        )
        self.button.clicked.connect(self.open_file_dialog)

        layout = QVBoxLayout()
        layout.addWidget(self.button)
        self.setLayout(layout)

    def open_file_dialog(self) -> None:
        """Open a file dialog based on the selected mode."""
        if self.mode == "open":
            file_name, _ = QFileDialog.getOpenFileName(
                self,
                self.title,
                self.directory,
                "All Files (*);;Text Files (*.txt)",
            )
        else:
            file_name, _ = QFileDialog.getSaveFileName(
                self,
                self.title,
                self.directory,
                "All Files (*);;Text Files (*.txt)",
            )

        if file_name:
            self.selectedFile.emit(file_name)



"""Collection of models for PyQt6"""

from collections.abc import Callable
from dataclasses import dataclass

from PyQt6.QtCore import QSize, QObject
from PyQt6.QtGui import QIcon, QColor, QAction

from jbconsts import COLORS
from jbutils.models import Base

from jbqt.consts import ICONS, QtIconSizes
from jbqt.models import model_utils


@dataclass
class ToolbarButton(Base):
    name: str = ""
    icon: QIcon | str = ""
    color: QColor | str = COLORS.BLUE_GRAY
    checkable: bool = False
    checked: bool = False
    status_tip: str = ""
    size: QSize | str = QtIconSizes.ICON_MD
    function: Callable[..., None] | str | None = None
    text: str = ""

    def __post_init__(self) -> None:
        if isinstance(self.function, str):
            fct = model_utils.get_fct(self.function)
            print("fct:", fct)
            self.function = fct if fct else None

    def to_action(self, parent: QObject | None = None) -> QAction:
        color: QColor | str = self.color
        if isinstance(color, str):
            color = COLORS.get(color, color)

        size = self.size
        if isinstance(size, str):
            size = QtIconSizes.get(size)

        icon = self.icon
        if isinstance(icon, str):
            icon_getter = ICONS.get(icon)
            if callable(icon_getter):
                icon = icon_getter(color, size)

            icon = QIcon() if isinstance(icon, str) else icon

        action = QAction(icon=icon, text=self.text, parent=parent)
        if self.checkable:
            action.setCheckable(True)
            action.setChecked(self.checkable)
        if self.status_tip:
            action.setStatusTip(self.status_tip)
        if self.function:
            action.triggered.connect(self.function)

        return action



"""Model exports"""

from jbqt.models.chips import IChipsWidget
from jbqt.models.chip_button import IChipButton
from jbqt.models.dialog_options import DialogOptions
from jbqt.models.model_consts import RegisteredFunctions
from jbqt.models.model_utils import get_fct, register_fct
from jbqt.models.toolbar_button import ToolbarButton

__all__ = [
    "DialogOptions",
    "get_fct",
    "IChipButton",
    "IChipsWidget",
    "register_fct",
    "RegisteredFunctions",
    "ToolbarButton",
]
